
clear; 

% Set Random number generator seed.
rng(0); 

% Number of simulations
simnum=100000;

% Load Data
load Annual_Data.txt;
cons=Annual_Data(:,1);
div=Annual_Data(:,2);
pd=Annual_Data(:,3);
ret=Annual_Data(:,4);
rf=Annual_Data(:,5);
exret=ret-rf; 
[T K]=size(Annual_Data); 
yearly_months=T*12; 


% Basic moments in the data
moment_mat_y=[100*mean(cons) 100*std(cons) ACC(cons,1); 100*mean(div) 100*std(div) ACC(div,1);100*mean(ret) 100*std(ret) ACC(ret,1); 100*mean(rf) 100*std(rf) ...
    ACC(rf,1); mean(pd) std(pd) ACC(pd,1)]'; 

% Variance ratios in the data
AVc=var_ratio(cons,5);
AVd=var_ratio(div,5);
AVrf=var_ratio(rf,5);

% Predictive regressions in the data
[beta_cons t_cons r_cons]=year_hor_reg(cons,pd,1); 
[beta_div t_div r_div]=year_hor_reg(div,pd,1); 
[beta_ret t_ret r_ret]=year_hor_reg(exret,pd,1); 
[beta_rf t_rf r_rf]=year_hor_reg(rf,pd,1); 
predict_ret_y=[beta_ret t_ret r_ret];
predict_cons_y=[beta_cons t_cons r_cons];
predict_div_y=[beta_div t_div r_div];
predict_rf_y=[beta_rf t_rf r_rf]; 


% Generate matrices to store simulation output

% Simulated moments
Mom_y_benchmark=zeros(3,6,simnum); Mom_y_extended=zeros(3,6,simnum);

% Simulated equity premia
Equity_premia=zeros(8,1,simnum);

% Simulated variance ratios
con_vr_benchmark=zeros(5,simnum); con_vr_extended=zeros(5,simnum);
div_vr_benchmark=zeros(5,simnum); div_vr_extended=zeros(5,simnum);
rf_vr_benchmark=zeros(5,simnum); rf_vr_extended=zeros(5,simnum);

% Simulated predictive regressions
ret_y_benchmark=zeros(3,2,simnum); ret_y_extended=zeros(3,2,simnum);
cons_y_benchmark=zeros(3,2,simnum); cons_y_extended=zeros(3,2,simnum);
div_y_benchmark=zeros(3,2,simnum); div_y_extended=zeros(3,2,simnum);
rf_y_benchmark=zeros(3,2,simnum); rf_y_extended=zeros(3,2,simnum);

% Instances where variance is censored at zero
Indicator_ybenchmark=zeros(simnum,1); 
Indicator_yextended=zeros(simnum,1); 
numcens_ybenchmark=zeros(simnum,1); 
numcens_yextended=zeros(simnum,1); 


% Simulations
for i=1:simnum
    i

    % Generate simulated data
    % Benchmark model (model 1)
    [benchmark_pyf Indicator_ybenchmark(i,1) numcens_ybenchmark(i,1) ] = AELR_generate_y(yearly_months,1);
    % Extended model (model 2)
    [extended_pyf Indicator_yextended(i,1) numcens_yextended(i,1)] = AELR_generate_y(yearly_months,2);
    
    % Benchmark model without valuation risk (model 3)
    model3_pyf = AELR_generate_y(yearly_months,3);
    % Extended model without valuation risk (model 4)
    model4_pyf = AELR_generate_y(yearly_months,4);
    % Extended model without long-run risk (model 5)
    model5_pyf = AELR_generate_y(yearly_months,5);
    % Extended model without valuation risk or long-run risk (model 6)
    model6_pyf = AELR_generate_y(yearly_months,6);
    % Extended model without long-run risk, homoscedastic (model 7)
    model7_pyf = AELR_generate_y(yearly_months,7);
    % Extended model without valuation risk or long-run risk, homoscedastic (model 8)
    model8_pyf = AELR_generate_y(yearly_months,8);
    
    % Basic moments
    Mom_y_benchmark(:,:,i)=[100*mean(benchmark_pyf(:,1)) 100*std(benchmark_pyf(:,1)) ACC(benchmark_pyf(:,1),1); 100*mean(benchmark_pyf(:,2)) 100*std(benchmark_pyf(:,2)) ACC(benchmark_pyf(:,2),1);100*mean(benchmark_pyf(:,4))...
        100*std(benchmark_pyf(:,4)) ACC(benchmark_pyf(:,4),1); 100*mean(benchmark_pyf(:,5)) 100*std(benchmark_pyf(:,5)) ACC(benchmark_pyf(:,5),1); mean(benchmark_pyf(:,3)) ...
        std(benchmark_pyf(:,3)) ACC(benchmark_pyf(:,3),1);100*(mean(benchmark_pyf(:,6))+.5*var(benchmark_pyf(:,6))) 100*std(benchmark_pyf(:,6)) 0]'; 
    Mom_y_extended(:,:,i)=[100*mean(extended_pyf(:,1)) 100*std(extended_pyf(:,1)) ACC(extended_pyf(:,1),1); 100*mean(extended_pyf(:,2)) 100*std(extended_pyf(:,2)) ACC(extended_pyf(:,2),1);100*mean(extended_pyf(:,4))...
        100*std(extended_pyf(:,4)) ACC(extended_pyf(:,4),1); 100*mean(extended_pyf(:,5)) 100*std(extended_pyf(:,5)) ACC(extended_pyf(:,5),1); mean(extended_pyf(:,3)) ...
        std(extended_pyf(:,3)) ACC(extended_pyf(:,3),1);100*(mean(extended_pyf(:,6))+.5*var(extended_pyf(:,6))) 100*std(extended_pyf(:,6)) 0]'; 

    % Equity premia
    Equity_premia(:,:,i)=[100*(mean(benchmark_pyf(:,6))+.5*var(benchmark_pyf(:,6)));
        100*(mean(extended_pyf(:,6))+.5*var(extended_pyf(:,6)));
        100*(mean(model3_pyf(:,6))+.5*var(model3_pyf(:,6)));
        100*(mean(model4_pyf(:,6))+.5*var(model4_pyf(:,6)));
        100*(mean(model5_pyf(:,6))+.5*var(model5_pyf(:,6)));
        100*(mean(model6_pyf(:,6))+.5*var(model6_pyf(:,6)));
        100*(mean(model7_pyf(:,6))+.5*var(model7_pyf(:,6)));
        100*(mean(model8_pyf(:,6))+.5*var(model8_pyf(:,6)))];
    
    % Variance ratios
    con_vr_benchmark(:,i)=var_ratio(benchmark_pyf(:,1),5); div_vr_benchmark(:,i)=var_ratio(benchmark_pyf(:,2),5); rf_vr_benchmark(:,i)=var_ratio(benchmark_pyf(:,5),5); 
    con_vr_extended(:,i)=var_ratio(extended_pyf(:,1),5); div_vr_extended(:,i)=var_ratio(extended_pyf(:,2),5); rf_vr_extended(:,i)=var_ratio(extended_pyf(:,5),5); 

    % Predictive regressions - benchmark model
    [beta_cons_benchmark t_cons_benchmark r_cons_benchmark]=year_hor_reg(benchmark_pyf(:,1),benchmark_pyf(:,3),1); 
    [beta_div_benchmark t_div_benchmark r_div_benchmark]=year_hor_reg(benchmark_pyf(:,2),benchmark_pyf(:,3),1); 
    [beta_ret_benchmark t_ret_benchmark r_ret_benchmark]=year_hor_reg(benchmark_pyf(:,6),benchmark_pyf(:,3),1); 
    [beta_rf_benchmark t_rf_benchmark r_rf_benchmark]=year_hor_reg(benchmark_pyf(:,5),benchmark_pyf(:,3),1); 
    ret_y_benchmark(:,:,i)=[beta_ret_benchmark r_ret_benchmark];cons_y_benchmark(:,:,i)=[beta_cons_benchmark r_cons_benchmark]; div_y_benchmark(:,:,i)=[beta_div_benchmark r_div_benchmark]; rf_y_benchmark(:,:,i)=[beta_rf_benchmark r_rf_benchmark]; 

    % Predictive regressions - extended model
    [beta_cons_extended t_cons_extended r_cons_extended]=year_hor_reg(extended_pyf(:,1),extended_pyf(:,3),1); 
    [beta_div_extended t_div_extended r_div_extended]=year_hor_reg(extended_pyf(:,2),extended_pyf(:,3),1); 
    [beta_ret_extended t_ret_extended r_ret_extended]=year_hor_reg(extended_pyf(:,6),extended_pyf(:,3),1); 
    [beta_rf_extended t_rf_extended r_rf_extended]=year_hor_reg(extended_pyf(:,5),extended_pyf(:,3),1); 
    ret_y_extended(:,:,i)=[beta_ret_extended r_ret_extended];cons_y_extended(:,:,i)=[beta_cons_extended r_cons_extended]; div_y_extended(:,:,i)=[beta_div_extended r_div_extended]; rf_y_extended(:,:,i)=[beta_rf_extended r_rf_extended]; 
end


% Percentage of simularions with censoring

Censor_vector=zeros(2,1); 
Censor_vector(1,1)=mean(Indicator_ybenchmark);
Censor_vector(2,1)= mean(Indicator_yextended); 


% Simulated moments

Med_mom_y_benchmark=zeros(3,6);
Med_mom_y_extended=zeros(3,6); 

for i=1:3
for j=1:6    
    Med_mom_y_benchmark(i,j)=quantile(Mom_y_benchmark(i,j,:),.5);   
    Med_mom_y_extended(i,j)=quantile(Mom_y_extended(i,j,:),.5);   
end
end


% Simulated equity premia

Med_equity_premia=quantile(Equity_premia,.5,3);


% Simulated variance ratios

cons_VR_50_benchmark=zeros(5,1); div_VR_50_benchmark=zeros(5,1); cons_VR_50_extended=zeros(5,1); div_VR_50_extended=zeros(5,1);
cons_VR_perc_benchmark=zeros(5,1); div_VR_perc_benchmark=zeros(5,1); cons_VR_perc_extended=zeros(5,1); div_VR_perc_extended=zeros(5,1); 

for i=1:5
    cons_VR_50_benchmark(i,1)=quantile(con_vr_benchmark(i,:),.5); 
    cons_VR_50_extended(i,1)=quantile(con_vr_extended(i,:),.5); 
    div_VR_50_benchmark(i,1)=quantile(div_vr_benchmark(i,:),.5); 
    div_VR_50_extended(i,1)=quantile(div_vr_extended(i,:),.5); 
    rf_VR_50_benchmark(i,1)=quantile(rf_vr_benchmark(i,:),.5); 
    rf_VR_50_extended(i,1)=quantile(rf_vr_extended(i,:),.5); 

    cons_VR_perc_benchmark(i,1)=(find([-1000; sort(reshape(con_vr_benchmark(i,:),simnum,1))]<AVc(i,1), 1, 'last' )-1)/simnum; 
    cons_VR_perc_extended(i,1)=(find([-1000; sort(reshape(con_vr_extended(i,:),simnum,1))]<AVc(i,1), 1, 'last' )-1)/simnum; 
    div_VR_perc_benchmark(i,1)=(find([-1000; sort(reshape(div_vr_benchmark(i,:),simnum,1))]<AVd(i,1), 1, 'last' )-1)/simnum; 
    div_VR_perc_extended(i,1)=(find([-1000; sort(reshape(div_vr_extended(i,:),simnum,1))]<AVd(i,1), 1, 'last' )-1)/simnum; 
    rf_VR_perc_benchmark(i,1)=(find([-1000; sort(reshape(rf_vr_benchmark(i,:),simnum,1))]<AVrf(i,1), 1, 'last' )-1)/simnum; 
    rf_VR_perc_extended(i,1)=(find([-1000; sort(reshape(rf_vr_extended(i,:),simnum,1))]<AVrf(i,1), 1, 'last' )-1)/simnum;     
end


% Simulated predictive regressions

Ret_50_benchmark_y_b=zeros(3,1); Cons_50_benchmark_y_b=zeros(3,1); Div_50_benchmark_y_b=zeros(3,1); Rf_50_benchmark_y_b=zeros(3,1);
Ret_50_extended_y_b=zeros(3,1); Cons_50_extended_y_b=zeros(3,1); Div_50_extended_y_b=zeros(3,1); Rf_50_extended_y_b=zeros(3,1); 

Ret_perc_benchmark_y_b=zeros(3,1); Cons_perc_benchmark_y_b=zeros(3,1); Div_perc_benchmark_y_b=zeros(3,1); Rf_perc_benchmark_y_b=zeros(3,1); 
Ret_perc_extended_y_b=zeros(3,1); Cons_perc_extended_y_b=zeros(3,1); Div_perc_extended_y_b=zeros(3,1); Rf_perc_extended_y_b=zeros(3,1); 

Ret_50_benchmark_y_r=zeros(3,1); Cons_50_benchmark_y_r=zeros(3,1); Div_50_benchmark_y_r=zeros(3,1); Rf_50_benchmark_y_r=zeros(3,1); 
Ret_50_extended_y_r=zeros(3,1); Cons_50_extended_y_r=zeros(3,1); Div_50_extended_y_r=zeros(3,1); Rf_50_extended_y_r=zeros(3,1); 

Ret_perc_benchmark_y_r=zeros(3,1); Cons_perc_benchmark_y_r=zeros(3,1); Div_perc_benchmark_y_r=zeros(3,1); Rf_perc_benchmark_y_r=zeros(3,1); 
Ret_perc_extended_y_r=zeros(3,1); Cons_perc_extended_y_r=zeros(3,1); Div_perc_extended_y_r=zeros(3,1); Rf_perc_extended_y_r=zeros(3,1); 

for i=1:3
    Ret_50_benchmark_y_b(i,1)=quantile(ret_y_benchmark(i,1,:),.5); 
    Cons_50_benchmark_y_b(i,1)=quantile(cons_y_benchmark(i,1,:),.5); 
    Div_50_benchmark_y_b(i,1)=quantile(div_y_benchmark(i,1,:),.5); 
    Rf_50_benchmark_y_b(i,1)=quantile(rf_y_benchmark(i,1,:),.5); 

    Ret_50_benchmark_y_r(i,1)=quantile(ret_y_benchmark(i,2,:),.5); 
    Cons_50_benchmark_y_r(i,1)=quantile(cons_y_benchmark(i,2,:),.5); 
    Div_50_benchmark_y_r(i,1)=quantile(div_y_benchmark(i,2,:),.5); 
    Rf_50_benchmark_y_r(i,1)=quantile(rf_y_benchmark(i,2,:),.5); 

    Ret_50_extended_y_b(i,1)=quantile(ret_y_extended(i,1,:),.5); 
    Cons_50_extended_y_b(i,1)=quantile(cons_y_extended(i,1,:),.5); 
    Div_50_extended_y_b(i,1)=quantile(div_y_extended(i,1,:),.5); 
    Rf_50_extended_y_b(i,1)=quantile(rf_y_extended(i,1,:),.5); 

    Ret_50_extended_y_r(i,1)=quantile(ret_y_extended(i,2,:),.5); 
    Cons_50_extended_y_r(i,1)=quantile(cons_y_extended(i,2,:),.5); 
    Div_50_extended_y_r(i,1)=quantile(div_y_extended(i,2,:),.5); 
    Rf_50_extended_y_r(i,1)=quantile(rf_y_extended(i,2,:),.5); 

    Ret_perc_benchmark_y_b(i,1)=(find([-1000; sort(reshape(ret_y_benchmark(i,1,:),simnum,1,1))]<predict_ret_y(i,1), 1, 'last' )-1)/simnum; 
    Cons_perc_benchmark_y_b(i,1)=(find([-1000; sort(reshape(cons_y_benchmark(i,1,:),simnum,1,1))]<predict_cons_y(i,1), 1, 'last' )-1)/simnum; 
    Div_perc_benchmark_y_b(i,1)=(find([-1000; sort(reshape(div_y_benchmark(i,1,:),simnum,1,1))]<predict_div_y(i,1), 1, 'last' )-1)/simnum; 
    Rf_perc_benchmark_y_b(i,1)=(find([-1000; sort(reshape(rf_y_benchmark(i,1,:),simnum,1,1))]<predict_rf_y(i,1), 1, 'last' )-1)/simnum; 

    Ret_perc_benchmark_y_r(i,1)=(find([-1000; sort(reshape(ret_y_benchmark(i,2,:),simnum,1,1))]<predict_ret_y(i,3), 1, 'last' )-1)/simnum; 
    Cons_perc_benchmark_y_r(i,1)=(find([-1000; sort(reshape(cons_y_benchmark(i,2,:),simnum,1,1))]<predict_cons_y(i,3), 1, 'last' )-1)/simnum; 
    Div_perc_benchmark_y_r(i,1)=(find([-1000; sort(reshape(div_y_benchmark(i,2,:),simnum,1,1))]<predict_div_y(i,3), 1, 'last' )-1)/simnum; 
    Rf_perc_benchmark_y_r(i,1)=(find([-1000; sort(reshape(rf_y_benchmark(i,2,:),simnum,1,1))]<predict_rf_y(i,3), 1, 'last' )-1)/simnum; 

    Ret_perc_extended_y_b(i,1)=(find([-1000; sort(reshape(ret_y_extended(i,1,:),simnum,1,1))]<predict_ret_y(i,1), 1, 'last' )-1)/simnum; 
    Cons_perc_extended_y_b(i,1)=(find([-1000; sort(reshape(cons_y_extended(i,1,:),simnum,1,1))]<predict_cons_y(i,1), 1, 'last' )-1)/simnum; 
    Div_perc_extended_y_b(i,1)=(find([-1000; sort(reshape(div_y_extended(i,1,:),simnum,1,1))]<predict_div_y(i,1), 1, 'last' )-1)/simnum; 
    Rf_perc_extended_y_b(i,1)=(find([-1000; sort(reshape(rf_y_extended(i,1,:),simnum,1,1))]<predict_rf_y(i,1), 1, 'last' )-1)/simnum; 

    Ret_perc_extended_y_r(i,1)=(find([-1000; sort(reshape(ret_y_extended(i,2,:),simnum,1,1))]<predict_ret_y(i,3), 1, 'last' )-1)/simnum; 
    Cons_perc_extended_y_r(i,1)=(find([-1000; sort(reshape(cons_y_extended(i,2,:),simnum,1,1))]<predict_cons_y(i,3), 1, 'last' )-1)/simnum; 
    Div_perc_extended_y_r(i,1)=(find([-1000; sort(reshape(div_y_extended(i,2,:),simnum,1,1))]<predict_div_y(i,3), 1, 'last' )-1)/simnum; 
    Rf_perc_extended_y_r(i,1)=(find([-1000; sort(reshape(rf_y_extended(i,2,:),simnum,1,1))]<predict_rf_y(i,3), 1, 'last' )-1)/simnum; 
end


% Tables

Table_3=[moment_mat_y(:,1) Med_mom_y_benchmark(:,1) Med_mom_y_extended(:,1);...
         moment_mat_y(:,2) Med_mom_y_benchmark(:,2) Med_mom_y_extended(:,2);...
         moment_mat_y(:,3) Med_mom_y_benchmark(:,3) Med_mom_y_extended(:,3);...
         moment_mat_y(:,4) Med_mom_y_benchmark(:,4) Med_mom_y_extended(:,4);...
         moment_mat_y(:,5) Med_mom_y_benchmark(:,5) Med_mom_y_extended(:,5)];
save Table3.txt Table_3 -ascii
     
Table_4=[Med_equity_premia(1) Med_equity_premia(2) NaN;...
         Med_equity_premia(3) Med_equity_premia(4) NaN;...
         NaN Med_equity_premia(5) Med_equity_premia(7);...
         NaN Med_equity_premia(6) Med_equity_premia(8)];
save Table4.txt Table_4 -ascii
     
Table_5=[AVc(1,1) cons_VR_50_benchmark(1) 1-cons_VR_perc_benchmark(1) cons_VR_50_extended(1) 1-cons_VR_perc_extended(1) ; ...
    AVc(3,1) cons_VR_50_benchmark(3) 1-cons_VR_perc_benchmark(3) cons_VR_50_extended(3) 1-cons_VR_perc_extended(3) ; ...
    AVc(5,1) cons_VR_50_benchmark(5) 1-cons_VR_perc_benchmark(5) cons_VR_50_extended(5) 1-cons_VR_perc_extended(5);...
    AVd(1,1) div_VR_50_benchmark(1) 1-div_VR_perc_benchmark(1) div_VR_50_extended(1) 1-div_VR_perc_extended(1);...
    AVd(3,1) div_VR_50_benchmark(3) 1-div_VR_perc_benchmark(3) div_VR_50_extended(3) 1-div_VR_perc_extended(3);...
    AVd(5,1) div_VR_50_benchmark(5) 1-div_VR_perc_benchmark(5) div_VR_50_extended(5) 1-div_VR_perc_extended(5);...
    AVrf(1,1) rf_VR_50_benchmark(1) 1-rf_VR_perc_benchmark(1) rf_VR_50_extended(1) 1-rf_VR_perc_extended(1);...
    AVrf(3,1) rf_VR_50_benchmark(3) 1-rf_VR_perc_benchmark(3) rf_VR_50_extended(3) 1-rf_VR_perc_extended(3);...
    AVrf(5,1) rf_VR_50_benchmark(5) 1-rf_VR_perc_benchmark(5) rf_VR_50_extended(5) 1-rf_VR_perc_extended(5)];
save Table5.txt Table_5 -ascii

% Table 6 output includes simulated coefficients reported in Table IA.1
Table_6=[predict_ret_y(:,1) predict_ret_y(:,2) predict_ret_y(:,3) Ret_50_benchmark_y_r 1-Ret_perc_benchmark_y_r Ret_50_extended_y_r 1-Ret_perc_extended_y_r Ret_50_benchmark_y_b 1-Ret_perc_benchmark_y_b Ret_50_extended_y_b 1-Ret_perc_extended_y_b;...
         predict_cons_y(:,1) predict_cons_y(:,2) predict_cons_y(:,3) Cons_50_benchmark_y_r 1-Cons_perc_benchmark_y_r Cons_50_extended_y_r 1-Cons_perc_extended_y_r Cons_50_benchmark_y_b 1-Cons_perc_benchmark_y_b Cons_50_extended_y_b 1-Cons_perc_extended_y_b;...
         predict_div_y(:,1) predict_div_y(:,2) predict_div_y(:,3) Div_50_benchmark_y_r 1-Div_perc_benchmark_y_r Div_50_extended_y_r 1-Div_perc_extended_y_r Div_50_benchmark_y_b 1-Div_perc_benchmark_y_b Div_50_extended_y_b 1-Div_perc_extended_y_b;...
         predict_rf_y(:,1) predict_rf_y(:,2) predict_rf_y(:,3) Rf_50_benchmark_y_r 1-Rf_perc_benchmark_y_r Rf_50_extended_y_r 1-Rf_perc_extended_y_r Rf_50_benchmark_y_b 1-Rf_perc_benchmark_y_b Rf_50_extended_y_b 1-Rf_perc_extended_y_b];
save Table6.txt Table_6 -ascii
