function [pi_star pi_hat pi_bar]=calculate_pi(model)

if model == 1
    %Benchmark model
    gamma = 1.516;
    psi = 1.4567;
    delta = .99795;
    sigma_c = 0.0069004;
    mu = 0.0015644;
    pi_cl = 0;
    sigma_d_input = 0.017103;
    pi_dc = 0.00049127;
    pi_dl = 0;
    sigma_eta = 0;
    rho_lambda = 0.99132;
    sigma_lambda = 0.00058631;
    nu = 0;
    sigma_omega = 0;
    alpha_c = 0;
    alpha_d = 0;
    rho_c = 0;
    rho_d = 0;

    mu_c = mu;
    mu_d = mu*(1-rho_d)/(1-rho_c);
    sigma = sigma_c;
    theta = (1-gamma)/(1-1/psi);
    sigma2 = sigma^2;
    
    sigma2_min = sigma2 - 0.000000001;
    sigma2_max = sigma2 + 0.000000001;
    delta_c_min = mu/(1-rho_c) - 0.000000001;
    delta_c_max = mu/(1-rho_c) + 0.000000001;

    x_min = -0.02;
    x_max = 0.02;

    grid_size_c = 3;
    grid_size_sigma = 3;
    grid_size_x = 501;

elseif model == 2
    %Extended model
    gamma = 2.3961;
    psi = 2.2107;
    delta = .99796;
    sigma_c = 0.0046033;
    mu = 0.0010428;
    pi_cl = -.0029185;
    sigma_d_input = 0.025274;
    pi_dc = -1.3205;
    pi_dl = -0.010907;
    sigma_eta = 0.0077828;
    rho_lambda = 0.99168;
    sigma_lambda = 0.00038594;
    nu = 0.99717;
    sigma_omega = 0.00000168;
    alpha_c = -95.3767;
    alpha_d = -151.8814;
    rho_c = 0.094708;
    rho_d = 0.33427;

    mu_c = mu;
    mu_d = mu*(1-rho_d)/(1-rho_c);
    sigma = sigma_c;
    theta = (1-gamma)/(1-1/psi);
    sigma2 = sigma^2;

    sigma2_min = sigma2 - 0.000075;
    sigma2_max = sigma2 + 0.000075;
    delta_c_min = mu/(1-rho_c) - 0.02;
    delta_c_max = mu/(1-rho_c) + 0.02;
    x_min = -0.02;
    x_max = 0.02;

    grid_size_c = 11;
    grid_size_sigma = 201;
    grid_size_x = 201;    
end


I_outer = 4;

delta_c_t = linspace(delta_c_min,delta_c_max,grid_size_c);
sigma2_t = linspace(sigma2_min,sigma2_max,grid_size_sigma)';
x_t = permute(linspace(x_min,x_max,grid_size_x), [3 1 2]);

J = ones(grid_size_sigma,grid_size_c,grid_size_x);

T = 5000;
for t = 1:T
    t
    
    if t > T-10
        I=10000;
    elseif t > T-250
        I=1000;
    elseif t > T-1000
        I=100;
    else
        I=12;
    end

    J_j = repmat(zeros(size(J)),1,1,1,I_outer);

    randomseed = random('unid',1000000);

    parfor j=1:I_outer
        rng(randomseed + j);
    for i=1:(I/I_outer)
        epsilon_c = random('norm',0,1,size(J));
        epsilon_eta = random('norm',0,1,size(J));
        epsilon_w = random('norm',0,1,size(J));
        epsilon_lambda = random('norm',0,1,size(J));

        sigma2_sim = sigma2 + nu*(sigma2_t-sigma2) + sigma_omega*epsilon_w;
        delta_c_sim = mu_c + rho_c*delta_c_t + alpha_c*(sigma2_sim-sigma2) ...
                + pi_cl*epsilon_lambda ...
                + max(sigma2_t,.0000000000001).^0.5.*epsilon_c;
        x_sim = rho_lambda*x_t + sigma_lambda*epsilon_lambda;
        delta_lambda_sim = x_sim + sigma_eta*epsilon_eta;
  
        delta_c_sim_index = round((min(max(delta_c_sim,delta_c_min),delta_c_max) - delta_c_min)/(delta_c_max - delta_c_min)*(grid_size_c-1)) + 1;
        sigma2_sim_index = round((min(max(sigma2_sim,sigma2_min),sigma2_max) - sigma2_min)/(sigma2_max - sigma2_min)*(grid_size_sigma-1)) + 1;
        x_sim_index = round((min(max(x_sim,x_min),x_max) - x_min)/(x_max - x_min)*(grid_size_x-1)) + 1;
        lin_index_sim = sub2ind([grid_size_sigma,grid_size_c,grid_size_x] ...
            ,sigma2_sim_index ...
            ,delta_c_sim_index ...
            ,x_sim_index);
        
        % Simulate H_(t+1)^(1-gamma)
        % All utility functions are scaled by 1-delta (which represents the same preferences)
        H_sim = (exp(delta_c_sim).^(1-1/psi).*((1-delta) + delta*exp(delta_lambda_sim).*J(lin_index_sim).^((1-1/psi)/(1-gamma)))).^((1-gamma)/(1-1/psi));
        
        % Calculate mean of simulations within parfor loop        
        J_j(:,:,:,j) = J_j(:,:,:,j) + H_sim/(I/I_outer);
    end
    end
    
    % Calculate mean of simulations across parfor loops
    J = mean(J_j,4);

    % Calculate matrix index corresponding to average state variables
    delta_c0_index = round((min(max(mu/(1-rho_c),delta_c_min),delta_c_max) - delta_c_min)/(delta_c_max - delta_c_min)*(grid_size_c-1)) + 1;
    sigma20_index = round((min(max(sigma2,sigma2_min),sigma2_max) - sigma2_min)/(sigma2_max - sigma2_min)*(grid_size_sigma-1)) + 1;
    x0_index = round((min(max(0,x_min),x_max) - x_min)/(x_max - x_min)*(grid_size_x-1)) + 1;

    % Calculate U0 at average state variables
    % Assumes C_t=1, Lambda_(t+t)=1, and Delta_c_t is long-run average
    U0 = (1/exp(mu/(1-rho_c)))*((1-delta)*exp(mu/(1-rho_c))^(1-1/psi) + delta*exp(mu/(1-rho_c))^(1-1/psi)*J(sigma20_index,delta_c0_index,x0_index)^((1-1/psi)/(1-gamma)))^(1/(1-1/psi));
end


%%%%%%%%%%%%%%%%%%%%%%%%%%
% Without Valuation Risk %
%%%%%%%%%%%%%%%%%%%%%%%%%%

I_outer = 4;

delta_c_t = linspace(delta_c_min,delta_c_max,grid_size_c);
sigma2_t = linspace(sigma2_min,sigma2_max,grid_size_sigma)';

J = ones(grid_size_sigma,grid_size_c);

T = 5000;
for t = 1:T
    t
    
    if t > T-10
        I=10000;
    elseif t > T-250
        I=1000;
    elseif t > T-1000
        I=100;
    else
        I=12;
    end
    
    J_j = repmat(zeros(size(J)),1,1,I_outer);

    randomseed = random('unid',1000000);

    parfor j=1:I_outer
        rng(randomseed + j);
    for i=1:(I/I_outer)
        epsilon_c = random('norm',0,1,size(J));
        epsilon_w = random('norm',0,1,size(J));
        epsilon_lambda = random('norm',0,1,size(J));

        sigma2_sim = sigma2 + nu*(sigma2_t-sigma2) + sigma_omega*epsilon_w;
        delta_c_sim = mu_c + rho_c*delta_c_t + alpha_c*(sigma2_sim-sigma2) ...
                + pi_cl*epsilon_lambda ...
                + max(sigma2_t,.0000000000001).^0.5.*epsilon_c;

        delta_c_sim_index = round((min(max(delta_c_sim,delta_c_min),delta_c_max) - delta_c_min)/(delta_c_max - delta_c_min)*(grid_size_c-1)) + 1;
        sigma2_sim_index = round((min(max(sigma2_sim,sigma2_min),sigma2_max) - sigma2_min)/(sigma2_max - sigma2_min)*(grid_size_sigma-1)) + 1;
        lin_index_sim = sub2ind([grid_size_sigma,grid_size_c] ...
            ,sigma2_sim_index ...
            ,delta_c_sim_index); 
        
        % Simulate H_(t+1)^(1-gamma)
        % All utility functions are scaled by 1-delta (which represents the same preferences)
        H_sim = (exp(delta_c_sim).^(1-1/psi).*((1-delta) + delta*J(lin_index_sim).^((1-1/psi)/(1-gamma)))).^((1-gamma)/(1-1/psi));
        
       % Calculate mean of simulations within parfor loop        
        J_j(:,:,j) = J_j(:,:,j) + H_sim/(I/I_outer);
    end
    end
    
    % Calculate mean of simulations across parfor loops
    J=mean(J_j,3);

    % Calculate matrix index corresponding to average state variables
    delta_c0_index = round((min(max(mu/(1-rho_c),delta_c_min),delta_c_max) - delta_c_min)/(delta_c_max - delta_c_min)*(grid_size_c-1)) + 1;
    sigma20_index = round((min(max(sigma2,sigma2_min),sigma2_max) - sigma2_min)/(sigma2_max - sigma2_min)*(grid_size_sigma-1)) + 1;

    % Calculate U0_hat at average state variables
    U0_hat = (1/exp(mu/(1-rho_c)))*((1-delta)*exp(mu/(1-rho_c))^(1-1/psi) + delta*exp(mu/(1-rho_c))^(1-1/psi)*J(sigma20_index,delta_c0_index)^((1-1/psi)/(1-gamma)))^(1/(1-1/psi));
end

pi_hat = 1 - U0/U0_hat;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Immediate Resolution of All Risk %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

T = 5000;
I = 100000;

C_t = ones(T,I);
sigma2_t = ones(T,I);
lambda_t = ones(T,I);
x_t = ones(T,I);
discount_t = ones(T,1);

C_lag = ones(1,I);
delta_c_lag = zeros(1,I);
sigma2_lag = ones(1,I)*sigma2;
lambda_lag = ones(1,I);
x_lag = zeros(1,I);
discount_lag = delta^-1;

for t=1:T
    epsilon_lambda = random('norm',0,1,[1,I]);
    epsilon_c = random('norm',0,1,[1,I]);
    epsilon_omega = random('norm',0,1,[1,I]);
    epsilon_eta = random('norm',0,1,[1,I]);
    
    sigma2_t(t,:) = sigma2 + nu*(sigma2_lag - sigma2) + sigma_omega.*epsilon_omega;
    C_t(t,:) = exp(log(C_lag) + mu_c + rho_c*delta_c_lag + alpha_c*(sigma2_t(t,:) - sigma2) ...
        + pi_cl*epsilon_lambda + max(sigma2_lag,.0000000000001).^(0.5).*epsilon_c);
    lambda_t(t,:) = exp(log(lambda_lag) + x_lag + sigma_eta*epsilon_eta);
    x_t(t,:) = rho_lambda*x_lag + sigma_lambda*epsilon_lambda;
    discount_t(t,1) = discount_lag*delta;
    
    delta_c_lag = log(C_t(t,:)./C_lag);
    C_lag = C_t(t,:);
    x_lag = x_t(t,:);
    sigma2_lag = sigma2_t(t,:);
    lambda_lag = lambda_t(t,:);
    discount_lag = discount_t(t,1);
end

U1_star = ((1-delta)*[discount_t',delta^T/(1-delta)]*([lambda_t.^(1/(1-1/psi)).*C_t;lambda_t(T,:).^(1/(1-1/psi)).*C_t(T,:).*repmat(U0,1,I)].^(1-1/psi))).^(1/(1-1/psi));
U0_star = ((1-delta) + delta*mean(U1_star.^(1-gamma))^((1-1/psi)/(1-gamma)))^(1/(1-1/psi));

pi_star = 1 - U0/U0_star;


%%%%%%%%%%%
% No Risk %
%%%%%%%%%%%

C_t_mean = mean(C_t,2);

U1_bar = ((1-delta)*[discount_t',delta^T/(1-delta)]*([C_t_mean.^(1-1/psi);(C_t_mean(T).*U0).^(1-1/psi)])).^(1/(1-1/psi));
U0_bar = ((1-delta) + delta*U1_bar^(1-1/psi))^(1/(1-1/psi));

pi_bar = 1 - U0/U0_bar;

end